#!/usr/bin/perl

###############################################
# TargetFinder - a program for genrating output file for functional annotation of cDNA sequences
# # copyright @ Jack Min
# 
# Note: version6.0 - code restructured.
#
########################## Body of Code ######################################
# the number of argument must be 5.

if ($#ARGV != 4)
{
	print "\nPlease check your input carefully!\n\n";
	print "usage: TargetIdentifier.pl  BlastOutputFile seqFastaFile aceFile lucy.info outputFile\n\n";
 	print "\n";
	print "Input:\n";
	print "		BlastOutputFile - BLAST output\n";
	print "		seqFastaFile - sequences in FASTA format used for BLAST\n";
	print "		aceFile (optional) - a file from PHRAP or CAP containing assembly data\n";
	print "		lucy.info (optional) - a file containing the low quality sequence length trimmed by LUCY\n";
	print "Output:\n";
	print "		outputFile - output with annotation\n\n";

	print "Warning: for optional files, file names must be inputed, empty files could be used!\n\n";

	print "\n";
	exit;
}

########### Parameters #########
$query = "" ;  #query sequece: contig number or clone number
$hitStatus = 0; # default as no hit
$hitInfo ="";  #the first hit infor
$frame =0; #used for translation
$queryStartPos = 0;
$queryEndPos = 0;
$sbjctStartPos = 0;
$queryStartLetter ="";
$sbjctStartLetter ="";
$head = 0; #false
$hit = 0;#false
$firstHit = 0; #false;
@fullLengthArray = ();#including full-lenght, short-full-length, possible-full-length
@partialArray = ();
@ambiguousArray =();
@NRlistArray = (); # to store the non-redundant list including no-hit singleton and contigs and full-lenth singleton and contigs
                   # ambugous sequences will not be included,  but will be manually examined if it is a full-lenth, it then will be added manually.
                   # non-full-length will not be included.


@partialList = ();
@ambiguousList = (); #for ambiguous list which will be used for retrieve ambiguous sequence
@allHitList = (); # for retrieve all hit sequences
@notHitList = ();
$result = "";
$proteinName = "";
$fullySeqCount = 0;
$sbjctLength = 0;
$queryLength = 0;

$full_num = 0;
$sFull_num = 0;
$pFull_num = 0;
$partial_num = 0;
$ambiguous_num = 0;
$full_com = 0;
$hitCount = 0;

#########################################

$BlastOutputFile = $ARGV[0];
$dnaFile = $ARGV[1];

$TargetOutputFile = $ARGV[4];
$AceFile = $ARGV[2];                 #used to open .ace file
$goodRegionFile = $ARGV[3];


$allHitFile = $ARGV[1].allHit;

$noHitFile = $ARGV[1].noHit;

#############

print "In progress ...\n";
$midLine = 0; #the line below the "Query:" to show identity and similarity
############parameters used to analyze the blast e-value to divide the sequences with hits into
# several bin: (1) e-value <= 1e-30
#              (2) 1e-30 < e-value <= 1e-10
#              (3) 1e-10 < e-value <= 1e-5
#              (4) 1e-5 < e-value <= 0.1
#              (5) e>0.1
###################################################
$expect = 1;
$case1count = 0;
$case2count = 0;
$case3count = 0;
$case4count = 0;
$case5count = 0;
###################################################

$position1 = 0; # blank space
$position2 = 0; # delimiter position including ", (, ["
$noHitCount = 0;
$queryCount = 0;

open (INPUT, $dnaFile) || die ("Cannot open file $dnaFile\n\n");
$seqFlag =0; #to indicate the line is header or seqs, used for seq in multiple lines
$value ='';
$key = '';
$count =0; #to indicate the first ">"
while (<INPUT>)
{
	$line = $_; #$_, default input variable
	chomp ($line);
	if ($line =~ /^>/)
	{
		$count++;
		if ($count >1)
		{
			$seqH{$key} = $value;
			# print $key."\n".$value."\n";

			$value ='';
			$key ='';
		}
		$seqFlag = 1;
		@tmpA = split(/ /, $line);
		$key = substr($tmpA[0], 1);
	}
	else
	{
		if ($seqFlag ==1)
		{
			$value .= $line;
		}
	}
}
$seqH{$key} = $value;

close (INPUT);

open (GOODREGIONINFO, $goodRegionFile) || die ("Cannot open file $goodRegionFile\n\n");
@goodRegionContent = <GOODREGIONINFO>;
close GOODREGIONINFO;

open (ACEFILE, $AceFile) || die ("Cannot open file $AceFile\n\n");
@aceContent = <ACEFILE>;
close (ACEFILE);

$originalFrame = 0; # this value will be used to check the original frame value since negative value will be changed positive for $frame
$LengthFlag = 0; # this flag to show the line position of "Length"
$category = ''; # categories including "" (100 -95%), "homolgue to" (95 -85%), "similar to" (85 -60%), "weakly similar to" (<60%)
$direction = '';
$unknown_num =0;
$signature = '';

open (BLASTOUTPUTFILE, $BlastOutputFile) || die ("Cannot open file $BlastOutputFile\n\n");

$offSet = tell(BLASTOUTPUTFILE);
$summaryStart = 0;
$hFlag = 0;  # to assume 'hypothetical protein' flag is off.
$sCount = 0;
@bContent = ();
#while (<BLASTOUTPUTFILE>)
while ($record = get_next_record(BLASTOUTPUTFILE))
{#foreachloop #1
	@recordA = split(/\n/, $record);
	$frame = 0;
	foreach $BlastContent (@recordA) #foreach start
	{
	 chomp ($BlastContent);
	@array1 = split (" ", $BlastContent);
	if ($array1[0] eq   "Query=")
	{
		$query = $array1[1];
		$queryCount++;
	#	print $queryCount;
		push (@allHitList, $query."\n");
 		if ($query !~ /Contig/)
		{
			@qArray1 = split(/\_/, $query);
			$signature = substr ($qArray1[0], 0, 4);
		}
		next;
	}
	
	if ($BlastContent =~ /No hits found/)
	{
	        push (@NRlistArray, $query."\n"); #store the no hit query in the NR-list
                $noHitCount++;
		push (@noHitList, $query."\n");
		$query = "";
              	@bContent = ();
		last;
	}
	elsif ($BlastContent =~ /^Sequences/)
	{
		$hit = 1; # there is a hit at least
		$hitCount++;
	}
	elsif ($BlastContent =~ /^>/)
	{
       		$hitInfo = $BlastContent;
             	$posT = index ($BlastContent, " ");
	     	$proteinName = substr ($BlastContent, $posT+1);
		$LengthFlag = 0;
		$hitFlag = 1;
		$midLine = 0;
		next;
	}
if ($hitFlag == 1)
{
        if ($BlastContent =~ /Length = /)
	{
		@Larray = split(/=/, $BlastContent);
		$sbjctLength = $Larray[1]*1;
		$LengthFlag = 1;
		next;
	}
	elsif ($BlastContent =~ /Expect/)
	{
		$hitInfo = $hitInfo.$BlastContent;
		@expectLine = split(/,/, $BlastContent);
		$eString = $expectLine[1];
		@subString = split (/ /, $eString);
		$expect = $subString[3];
		if ($expect > 1e-5)
		{
			$hitFlag = 0;
			last;
		}
	}
	elsif ($BlastContent =~ /Identities/)
	{
		@t1= split (/[()]/, $BlastContent);
		$identity = $t1[1] * 1;
	    
		if (($proteinName =~ /^probable/) || ($proteinName =~ /^Probable/) || ($proteinName =~ /^PROBABLE/) || ($proteinName =~ /^Putative/) || ($proteinName =~ /^putative/) || ($proteinName =~ /^PUTATIVE/))
	   	{
			$pos = index($proteinName, " ");
			$proteinName = substr ($proteinName, $pos+1);
	   	}
		if (($proteinName =~ /similar to/) || ($proteinName =~ /Similar to/) || ($proteinName =~ /SIMILAR TO/) || ($proteinName =~ /[Hh]omologue to/) || ($proteinName =~ /HOMOLOGUE TO/) || ($proteinName =~ /[Rr]elated to/))
		{
			$pos2 = index ($proteinName, "to");
                        $proteinName = substr ($proteinName, $pos2 + 3);
 		}
	      #################################
		if ($proteinName =~ /\[/)
		{
			$pos2 = index ($proteinName, "[");
			$proteinName = substr ($proteinName, 0, $pos2-1);
		}
		if ($proteinName =~ /gi\|/)
            	{
            		$position2 = index ($proteinName, "|");
	   		$proteinName = substr ($proteinName, 0, $position2 -1);
		}

		@tProtein = split(/\s+/, $proteinName);
		$proteinName = '';
		foreach $w (@tProtein)
		{
			$proteinName =$proteinName." ".$w;
		}
	}
	elsif ($BlastContent =~ /Frame/)
	{
		$frame = $array1[2];
		$frameCount++;
		#	print "frameCount: ".$frameCount."\n";
		$hitInfo = $hitInfo.$BlastConten;
		$originalFrame = $frame;
	}
	elsif ($BlastContent=~ /Query:/)
	{
		$queryStartPos = $array1[1];
		$queryEndPos = $array1[3];
		$queryStartLetter = substr($array1[2], 0, 1);
                $midLine = 1; # set it to true
	}
	elsif ($BlastContent=~ /Sbjct:/) #elsif -1
	{
		$sbjctStartPos = $array1[1];
		$sbjctEndPos = $array1[3];
		$sbjctStartLetter = substr($array1[2], 0, 1);
		$in = $proteinName."\t".$expect."\t".$identity."\t".$frame."\t".$queryStartPos."\t".$queryEndPos."\t".$sbjctStartPos."\t".$sbjctEndPos."\t".$sbjctLength."\t".$hitInfo."\n";
		###### save the parameters ##########
		push (@bContent, $in);
		$hitFlag = 0;
	}
	else
	{
		if ($LengthFlag == 0) #for multiple definition line)
		{
		#$BlastContent = s/\s//g;
		$proteinName = $proteinName." ".$BlastContent;
		#print "pname: ".$proteinName."\n";
 		}
	}
	
	if ($midLine == 0)
        {
             $hitInfo =$hitInfo.$BlastContent;
        }	
} #end if ($hitFlag == 1)		
} #end foreach

$hFlag = 0;
$sCount =0;
$yFlag = 0; #non hypothetical sbjct			   
if (@bContent >0)
{
	foreach $in (@bContent)
	{
		chomp ($in);
		$sCount++;
		@tmp = split (/\t/, $in);
		$pName = lc ($tmp[0]);
						
		if (($pName =~ /hypothetical/) || ($pName =~ /unnamed/) || ($pName =~ /predicted/) || ($pName =~ /unknown/))
		{
			$hFlag = 1;
	        }
		else   #newly added in Version5
		{
			if ($yFlag == 0)
			{
				$noHyp = $sCount -1;
				$yFlag = 1;
				$hFlag = 0;
				last;
			}
		}
	} #end foreach
	$t++;
	print "hit: ".$t."\n";
	if ($hFlag == 1)
	{
		$use = $bContent[0];
	}
	else
	{
		$use = $bContent[$noHyp];
	}
	@tmp = split (/\t/, $use);
	
	#$proteinName."\t".$expect."\t".$identity."\t".$frame."\t".$queryStartPos."\t".$queryEndPos."\t".$sbjctStartPos."\t".$sbjctEndPos."\t".$sbjctLength."\t".$hitInfo."\n";
	
	$pName = $tmp[0];
	#print $pName."\n";
	$expect = $tmp[1];
	$frame = $tmp[3];
	#print "frame: ".$frame."\n";
	$queryStartPos = $tmp[4];
	$queryEndPos = $tmp[5];
	$sbjctStartPos = $tmp[6];
	$sbjctEndPos = $tmp[7];
	$sbjctLength = $tmp[8];
	$hitInfo = $tmp[9];
		
	if ($expect <= 1e-100)
	{
		$category = '';
		$case100count++;
	}
	elsif ($expect <= 1e-50)
	{
		$category = "Homolgous to ";
		$case50count++;
	}
	elsif ($expect <= 1e-30)
	{
		$category = 'Similar to ';
		$case30count++;
	}
	elsif ($expect <= 1e-10)
	{
		$category = "Similar to ";
		$case10count++;
	}
	elsif ($expect <= 1e-5)
	{
	   	$case5count++;
		$category = "Weakly similar to ";
	 #  next;
        }
	elsif (($expect > 1e-5) && ($expect <= 0.1))
	{
		$case1count++;
	}

	#$pName = $category.$pName;
	@tProtein = split(/\s+/, $pName);
	$pName = '';
	foreach $w (@tProtein)
	{
        	if ($w =~ /\w/)
                {
                	if ($pName eq '')
                        {
                        	$pName = $w;
                       	}
                        else
                        {
                        	$pName =$pName." ".$w;
                        }        
		}
	}
	
   ############################################# this part for blastn ###########
	if ($frame == 0)
	{
		if ($query =~ /lucy.seq.Contig/)
		{
			@qArray = split(/\./, $query);
			$query = $qArray[2];
		}
		$result = $query."\t".$pName."\t"."N/A"."\t"."N/A"."\n";
		push (@partialArray, $result);
		push (@partialList, $query."\n");
		$result = "";
		$pName ='';
	}
	else #else A
	{
		$sequence = $seqH{$query};
		$wholeLength = length ($sequence);
		#to obtain the length of the whole sequence for frame with negative value
		$sequence = uc($sequence);
		$sbjctFlag = 0; # to compare the subject length with query
		if ($frame < 0)
		{
			$direction = "antisense";
		}
		elsif ($frame >0)
		{
			$direction = "sense";
		}
	
		if ($frame > 0)
		{
			if ((($sbjctLength - $sbjctStartPos) -($wholeLength - $queryStartPos)/3) > 0)
			{
				$sbjctFlag = 1; #partial
			}
		}
		elsif ($frame < 0)
		{
			$revComSeq = reverse $sequence;
			$revComSeq =~ tr/ACGTacgt/TGCAtgca/;
			$sequence = $revComSeq;
			$queryStartPos = $wholeLength - $queryStartPos +1;
			$queryEnd = $queryEndPos;
			$queryEndPos = $wholeLength - $queryEndPos +1;
			$originalFrame = $frame;
			$frame = $frame * (-1);
			if ((($sbjctLength - $sbjctEndPos) - $queryEnd/3) > 0)
			{
				$sbjctFlag = 1; #partial
			}
		}
		$subSeq = substr($sequence, $frame - 1, $queryEndPos);
		$length = length ($subSeq);
		@base = split(//, $subSeq);
		$index = 0;
		$stopCodonFlag = 0; # no stop codon
		$subSeq2 = substr ($sequence, $queryEndPos); #to avoid the first part that may have a stop codon
		$length2 = length ($subSeq2);
		@base2 = split(//, $subSeq2);
		$stopAfterStart = 0;
		$query = shortenID($query);
		$stopAfterStart = searchStopCodon(@base2);
		
		if  ($sbjctFlag ==0)
		{
			$info = $direction." complete";
		}
		else
		{
			$info = $direction." partial";
		}

		for ($index = 0; $index <= $length-3; $index++)
		{
			if(($base[$index] eq 'A') && ($base[$index + 1] eq 'T') && ($base[$index + 2] eq 'G'))
			{
                        	$full = 1;
                        	if ($stopCodonFlag == 1)
                                {
                                	if ($stopAfterStart ==1 && $sbjctFlag ==0)
                                	{
                                   		#$result = $query."\t". $pName."\t"."Full-length"."\t".$info."\n";
						$result = $pName."\t".$query."\t".$expect."\t"."FULL-LENGTH"."\t".$info."\t".$hitInfo."\n";	
						$fullySeqCount++;
						$full_num++;
						$full_com++;
						$stopAfterStart = 0;
					}
                                   	else
                                   	{
                                   		#$result = $query."\t".$pName."\t"."Full-length\t".$info."\n";
                                   		$result = $pName."\t".$query."\t".$expect."\t"."FULL-LENGTH"."\t".$info."\t".$hitInfo."\n";	
						$full_num++;
					}	
                                   	push (@fullLengthArray, $result);
                                   	push (@NRlistArray, $query."\n");
                                   	$result = "";
				   	$stopAfterStart = 0;
				 }
                                 else #stop codon before start codon is false
                                 {
					$shorter =   $sbjctStartPos - ($queryStartPos - $index - $frame)/3;
					$seqID = "";
                                    	if ($shorter < 10)
                                    	{
                                        	#$result = $query."\t".$pName."\t"."Full-length"."\t".$info."\n";
                                        	$result = $pName."\t".$query."\t".$expect."\t"."FULL-LENGTH"."\t".$info."\t".$hitInfo."\n";	
						$full_num++;
						push (@fullLengthArray, $result);
                                        	push (@NRlistArray, $query."\n");
                                        	$result ="";
						if ($stopAfterStart == 1)
                                        	{
				  			$fullySeqCount++;
                                        	}
                                     	}
					elsif (($shorter >= 10) && ($shorter <100))
                                	{
						#$result = $query."\t".$pName."\t"."Short ($shorter) full-length"."\t".$info."\n";
                                         	$result = $pName."\t".$query."\t".$expect."\t"."SHORT ($shorter) FULL-LENGTH"."\t".$info."\t".$hitInfo."\n";
						$sFull_num++;
						push (@fullLengthArray, $result);
                                         	push (@NRlistArray, $query."\n");
                                         	$result = "";
						if ($stopAfterStart == 1)
                                        	{
				  			$fullySeqCount++;
                                        	}
					}
					else # checking possible-full-length ###########
				 	{#start else-A
						$partialFlag = 0;
						if ($originalFrame < 0)
						{
							$partialFlag = 2;
						}
						elsif ($originalFrame >0)
						{#???
							if ($query =~ /Contig/) # for contig
							{
								foreach $line (@aceContent)
					  			{
									if ($line =~ /$query/)
									{
										$flagTmp = 1;
						   				next;
					        			}
									elsif ($line =~ /$signature/)
									{
										if ($flagTmp ==1)
						   				{
											@arrayTmp = split (/ /, $line);
											$seqID = $arrayTmp[1];
											last;
						   				}
									}
									else
									{
						 				next;
                                                			}
   					   			}# end foreach
					 		}
							else
							{
								$seqID = $query;
							}
					
	   						$trimLength = 0;
							foreach $seqLine (@goodRegionContent)
							{
					   			if ($seqLine =~ /$seqID/)
					   			{
									@arrayTmp3 = split (/\t/, $seqLine);
									$trimLength = $arrayTmp3[2];
					        			last;
					   			}
							}#end of foreach
							#compare the length
							$d = $trimLength - 60; 
							# spliced sequence from vector+adaptor is 60 nt
							if ($d <= 0)
							{
								$partialFlag = 1; #set to true
                     					}
							else
                                        		{
								if ($frame == 0)
								{
									$diff = ($d +$queryStartPos -$sbjctStartPos);
								}
								else
								{
						 			$diff =($d + $queryStartPos)/3 - $sbjctStartPos;
								}
								if ($diff > 0)
								{
									$partialFlag = 0; # possible full length
                                                		}
                                                		else
								{ $partialFlag = 1;}
                                        		}
        		         		} # end ??
        					if ($partialFlag ==0)
                                        	{
							#$results = $query."\t".$pName."\t"."Possible full-length"."\t".$info."\n";
							$results = $pName."\t".$query."\t".$expect."\t"."POSSIBLE FULL-LENGTH"."\t".$info."\t".$hitInfo."\n";
							push (@fullLengthArray, $results);
					  		$pFull_num++;
					  		$possibleCount++;
							push (@NRlistArray, $query."\n");
					  		$result = "";
				          		if ($stopAfterStart == 1)
					  		{
								$fullySeqCount++;
				          		}
						}
						elsif ($partialFlag ==1)
        					{
							#$result = $query."\t".$pName."\t"."Partial"."\t".$info."\n";
							$result = $pName."\t".$query."\t".$expect."\t"."PARTIAL"."\t".$info."\t".$hitInfo."\n";
							push (@partialArray, $result);
                        	       	    	        push (@partialList, $query."\n");
							$partial_num++;
							$result = "";
                                         	}
						else #if ($partialFlag ==2)
						{
							#$result = $query."\t".$pName."\t"."3'-sequenced partial"."\t".$info."\n";
							$result = $pName."\t".$query."\t".$expect."\t"."3'-SEQUENCED PARTIAL"."\t".$info."\t".$hitInfo."\n";
							push (@partialArray, $result);
                               	    	        	push (@partialList, $query."\n");
							$unknown_num++;
							$result = "";
					 	}
                                 	} #end else-A
				}# end of else $stop-codon-before start codon = false
                                $pName = "";
                                $query = "";
                                $hitInfo = "";
                                $stopAfterStart = 0;
				last;
			}
	 		elsif (($base[$index] eq 'T') && ($base[$index +1] eq 'A') && ($base[$index +2] eq 'A' ))
			{
				$stopCodonFlag = 1;
				$index = $index + 2;
			}
			elsif (($base[$index] eq 'T') && ($base[$index + 1] eq 'A') && ($base[$index +2] eq 'G'))
			{
				$stopCodonFlag = 1;
				$index = $index + 2;
			}
                        elsif (($base[$index] eq 'T') && ($base[$index +1] eq 'G') && ($base[$index +2] eq 'A'))
                        {
		   		$stopCodonFlag = 1;
                        	$index = $index +2;
			}
			else
                        {
                        	$index = $index + 2;
                        	$full = 0;
			}
                }#end for loop
		if ($full == 0)
		{
			$seqID = "";
			if ($stopCodonFlag ==1 && $originalFrame >0)
                	{
                                #$result = $query."\t".$pName."\t"."Ambiguous\t".$info."\n";
                                $result = $pName."\t".$query."\t".$expect."\t"."AMBIGUOUS\t".$info."\t".$hitInfo."\n";
				push (@ambiguousArray, $result);
                                push (@NRlistArray, $query."\n");
				push (@ambiguousList, $query."\n");
				$ambiguous_num++;
                                $result = "";
				$pName = "";
			}
                        else
			{# else -?
                        	$partialFlag = 0;
				if ($originalFrame < 0)
				{
					$partialFlag = 2;
				}
                                elsif ($originalFrame >0)
                                {#!
					if ($query =~ /Contig/) # for contig
					{
						foreach $line (@aceContent)
						{
							if ($line =~ /$query/)
							{
						   		$flagTmp = 1;
						   		next;
					        	}
							elsif ($line =~ /$signature/)
							{
								if ($flagTmp ==1)
						   		{
									@arrayTmp = split (/ /, $line);
									$seqID = $arrayTmp[1];
									last;
						   		}
                                                	}
					        	else
					        	{ next;
                                                        }
	                                	}#foreach
					 } #end if for contig
					else
					{
						$seqID = $query;
					}
					$trimLength = 0;
					foreach $seqLine (@goodRegionContent)
					{
					   if ($seqLine =~ /$seqID/)
					   {
						@arrayTmp3 = split (/\t/, $seqLine);
						$trimLength = $arrayTmp3[2];
					        last;
					   }
					   else
					   {
						next;
                                           }
					}#end of foreach
					#compare the length
					$d = $trimLength - 60; #
					if ($d <= 0)
					{
						$partialFlag = 1; #set to true
                     			}
					else
                                        {
						if  ($frame == 0)
						{
							$diff = $d + $queryStartPos - $sbjctStartPos;
						}
						else
						{
							$diff =($d + $queryStartPos)/3 - $sbjctStartPos;
						}

						if ($diff > 0)
						{
							$partialFlag = 0; # possible full length
                                                }
                                                else
						{ $partialFlag = 1;}
                                        }
        			   } # !
        			if ($partialFlag ==0)
                                {
					  #$results = $query."\t".$pName."\t"."Possible full-length"."\t".$info."\n";
					  $result = $pName."\t".$query."\t".$expect."\t"."POSSIBLE FULL-LENGTH"."\t".$info."\t".$hitInfo."\n";
					  push (@fullLengthArray, $result);
					  push (@NRlistArray, $query."\n");
                                          $possibleCount++;
					  $pFull_num++;
                                     #     print "RUN TO CHECK POSSIBLE FULL-LENGTH (2)...".$possibleCount."\n";
				          $result ="";
					  if ($stopAfterStart == 1)
                                          {
				  		$fullySeqCount++;
                                          }
				}
			       elsif ($partialFlag ==1)
                                {
					#$result = $query."\t".$proteinName."\t"."Partial"."\t".$info."\n";
					$result = $pName."\t".$query."\t".$expect."\t"."PARTIAL"."\t".$info."\t".$hitInfo."\n";
					push (@partialArray, $result);
                               	        push (@partialList, $query."\n");
					$partial_num++;
					$result = "";
                                }
				elsif ($partialFlag ==2)
				{
					#$result = $query."\t".$proteinName."\t"."3'-sequenced partial"."\t".$info."\n";
					$result = $pName."\t".$query."\t".$expect."\t"."3'-SEQUENCED PARTIAL"."\t".$info."\t".$hitInfo."\n";
					push (@partialArray, $result);
                               	        push (@partialList, $query."\n");
					$unknown_num++;
					$result = "";
				}
                        }     # end else-?
		} #end if ($full=0)
	}#end else A	
} #end if (@bContent >0)
@bContent = ();	      
$offSet = tell(BLASTOUTPUTFILE);
}#end of while
close (BLASTOUTPUTFILE);

open (TARGETOUTPUTFILE, ">$TargetOutputFile");
print TARGETOUTPUTFILE (@fullLengthArray);
print TARGETOUTPUTFILE (@ambiguousArray);
print TARGETOUTPUTFILE (@partialArray);

close (BLASTOUTPUTFILE);
close (TARGETOUTPUTFILE);

######## print out ambiguous list file and all-hit-file #########
#open (AMBIGUOUSFILE, ">$ambiguousFile") || die ("Can not open file $ambiguousFile for writing!\n\n");
#print AMBIGUOUSFILE (@ambiguousList);
#close AMBIGUOUSFILE;

open (ALLHITFILE, ">$allHitFile") || die ("Can not open file $allHitFile for writing!\n\n");
print ALLHITFILE (@allHitList);
close ALLHITFILE;


open (ACEFILE, $AceFile) || die ("Cannot open file $AceFile\n\n");

open (NOHIT, ">$noHitFile") || die ("Cannot open file for $noHitFile\n\n");
print NOHIT (@noHitList);
close NOHIT;


print "\n";
print "Summary of the Annotation:\n";
print "query count: ".$queryCount."\n";
print "no hit count: ".$noHitCount."\n";
print "hit count: ".$hitCount."\n";
print "full-length: ".$full_num."\n";
print "short full-length: ".$sFull_num."\n";
print "possible full-length: ".$pFull_num."\n";
print "partial: ".$partial_num."\n";
print "ambiguous: ".$ambiguous_num."\n";
print "3'-sequenced partial: ".$unknown_num."\n";
print "full-length & completely sequenced: ".$fullySeqCount."\n";

print "E-value <= 1e-100: ".$case100count."\n";
print "E-value <= 1e-50: ".$case50count."\n";
print "E-value <= 1e-30: ".$case30count."\n";
print "E-value <= 1e-10: ".$case10count."\n";
print "E-value <= 1e-5: ".$case5count."\n";
print "E-value <= 1e-1: ".$case1count."\n";

print "The work is done!!\n";
exit;

####################################################
# subroutine
# to change lucy.seq.Contig# into Contig# and Asn_###_XXXX.**** into Asn_###
#

sub shortenID
    {
	my ($s) = @_;

	use strict;
        my @t = ();
        my $s2;
	if ($s =~ /Contig/)
	{
           @t = split(/\./, $s);
           $s2 = $t[2];
        }

	else
        {
#	  @t = split (/_/, $s);
#          $s2 = $t[0]."_".$t[1];
	$s2 = $s;
	}

return $s2;
}

###################################################
#
# sub for searching stop codon
#
sub searchStopCodon
{
		my (@base2) = @_;
		use strict;
		my $length2 = scalar @base2;
		my $stopAfterStart = 0;

		for (my $index2 = 0; $index2 <= $length2 -3; $index2++)
                                 {
                                    if (($base2[$index2] eq 'T') && ($base2[$index2 +1] eq 'A') && ($base2[$index2 +2] eq 'A' ))
                                     {
                                        $stopAfterStart = 1;
                                     	last;
                                     }

			             elsif (($base2[$index2] eq 'T') && ($base2[$index2 + 1] eq 'A') && ($base2[$index2 +2] eq 'G'))
                                     {
                                        $stopAfterStart = 1;
                                        last;
                                     }
                                     elsif (($base2[$index2] eq 'T') && ($base2[$index2 +1] eq 'G') && ($base2[$index2 +2] eq 'A'))
                                     {
		   		        $stopAfterStart = 1;
                                        last;
                                     }

				     else
                                     {
                                  	$index2 = $index2 + 2;
                                        $stopAfterStart = 0;
                                     }
                                  }

			return $stopAfterStart;
}

###############
sub get_next_record
{
	my($fh) = @_;
	use strict;
	my($offSet);
	my($record);
	my($seperator) = $/;
	$/ = "BLASTX";
	$record =<$fh>;
	$/ =$seperator;
	return $record;
}
